// Generated by CoffeeScript 1.6.2
(function() {
  var Generator, Markdown, Referencer, Templater, fs, mkdirp, path, _;

  fs = require('fs');

  path = require('path');

  mkdirp = require('mkdirp');

  _ = require('underscore');

  Templater = require('./util/templater');

  Referencer = require('./util/referencer');

  Markdown = require('./util/markdown');

  module.exports = Generator = (function() {
    function Generator(parser, options) {
      this.parser = parser;
      this.options = options;
      this.referencer = new Referencer(this.parser.classes, this.parser.mixins, this.options);
      this.templater = new Templater(this.options, this.referencer, this.parser);
    }

    Generator.prototype.generate = function(file) {
      if (file) {
        this.templater.redirect(file);
      }
      this.generateIndex();
      this.generateClasses();
      this.generateMixins();
      this.generateFiles();
      this.generateExtras();
      this.generateClassMixinFileExtraIndex();
      this.generateClassAndMixinLists();
      this.generateMethodList();
      this.generateFileList();
      this.generateExtraList();
      this.generateSearchData(file);
      if (!(file || this.options.noOutput)) {
        return this.copyAssets();
      }
    };

    Generator.prototype.generateIndex = function() {
      var index, list;

      index = this.options.readme || 'class_index';
      list = this.parser.classes.length !== 0 ? 'class_list' : this.parser.files.length !== 0 ? 'file_list' : this.parser.mixins.length !== 0 ? 'mixin_list' : this.parser.getAllMethods().length !== 0 ? 'method_list' : 'extra_list';
      return this.templater.render('frames', {
        list: list,
        index: index,
        path: ''
      }, 'index.html');
    };

    Generator.prototype.generateClasses = function() {
      var assetPath, breadcrumbs, clazz, combined, delegation, method, methods, namespace, namespaces, originalStatus, _i, _j, _k, _l, _len, _len1, _len2, _len3, _ref, _ref1, _results,
        _this = this;

      _ref = this.parser.classes;
      _results = [];
      for (_i = 0, _len = _ref.length; _i < _len; _i++) {
        clazz = _ref[_i];
        namespaces = _.compact(clazz.getNamespace().split('.'));
        assetPath = '../';
        for (_j = 0, _len1 = namespaces.length; _j < _len1; _j++) {
          namespace = namespaces[_j];
          assetPath += '../';
        }
        breadcrumbs = [
          {
            href: "" + assetPath + "class_index.html",
            name: 'Index'
          }
        ];
        if (this.options.readme) {
          breadcrumbs.unshift({
            href: "" + assetPath + this.options.readme + ".html",
            name: this.options.name
          });
        }
        if (this.options.homepage) {
          breadcrumbs.unshift(this.options.homepage);
        }
        combined = [];
        for (_k = 0, _len2 = namespaces.length; _k < _len2; _k++) {
          namespace = namespaces[_k];
          combined.push(namespace);
          breadcrumbs.push({
            href: this.referencer.getLink(combined.join('.'), assetPath),
            name: namespace
          });
        }
        breadcrumbs.push({
          name: clazz.getName()
        });
        methods = clazz.getMethods();
        for (_l = 0, _len3 = methods.length; _l < _len3; _l += 1) {
          method = methods[_l];
          delegation = method.doc.delegation;
          if (delegation) {
            originalStatus = method.doc.status;
            _ref1 = this.referencer.resolveDelegation(method, delegation, clazz), method.doc = _ref1[0], method.parameters = _ref1[1];
            method.doc.status = originalStatus;
          }
        }
        _results.push(this.templater.render('class', {
          path: assetPath,
          classData: this.referencer.resolveDoc(clazz.toJSON(), clazz, assetPath),
          classMethods: _.map(_.filter(methods, function(method) {
            return method.type === 'class';
          }), function(m) {
            return _this.referencer.resolveDoc(m.toJSON(), clazz, assetPath);
          }),
          instanceMethods: _.map(_.filter(methods, function(method) {
            return method.type === 'instance';
          }), function(m) {
            return _this.referencer.resolveDoc(m.toJSON(), clazz, assetPath);
          }),
          properties: _.map(clazz.properties, function(p) {
            return _this.referencer.resolveDoc(p.toJSON(), clazz, assetPath);
          }),
          constants: _.map(_.filter(clazz.getVariables(), function(variable) {
            return variable.isConstant();
          }), function(m) {
            return _this.referencer.resolveDoc(m.toJSON(), clazz, assetPath);
          }),
          subClasses: _.map(this.referencer.getDirectSubClasses(clazz), function(c) {
            return c.getClassName();
          }),
          inheritedMethods: _.groupBy(this.referencer.getInheritedMethods(clazz), function(m) {
            return m.entity.getClassName();
          }),
          includedMethods: this.referencer.getIncludedMethods(clazz),
          extendedMethods: this.referencer.getExtendedMethods(clazz),
          concernMethods: this.referencer.getConcernMethods(clazz),
          inheritedConstants: _.groupBy(this.referencer.getInheritedConstants(clazz), function(m) {
            return m.entity.getClassName();
          }),
          inheritedProperties: _.groupBy(this.referencer.getInheritedProperties(clazz), function(m) {
            return m.entity.getClassName();
          }),
          breadcrumbs: breadcrumbs
        }, "classes/" + (clazz.getClassName().replace(/\./g, '/')) + ".html"));
      }
      return _results;
    };

    Generator.prototype.generateMixins = function() {
      var assetPath, breadcrumbs, combined, isExtendedFrom, isIncludedIn, mixin, namespace, namespaces, _i, _j, _k, _len, _len1, _len2, _ref, _results,
        _this = this;

      _ref = this.parser.mixins;
      _results = [];
      for (_i = 0, _len = _ref.length; _i < _len; _i++) {
        mixin = _ref[_i];
        namespaces = _.compact(mixin.getNamespace().split('.'));
        assetPath = '../';
        for (_j = 0, _len1 = namespaces.length; _j < _len1; _j++) {
          namespace = namespaces[_j];
          assetPath += '../';
        }
        breadcrumbs = [
          {
            href: "" + assetPath + "class_index.html",
            name: 'Index'
          }
        ];
        if (this.options.readme) {
          breadcrumbs.unshift({
            href: "" + assetPath + this.options.readme + ".html",
            name: this.options.name
          });
        }
        if (this.options.homepage) {
          breadcrumbs.unshift(this.options.homepage);
        }
        combined = [];
        for (_k = 0, _len2 = namespaces.length; _k < _len2; _k++) {
          namespace = namespaces[_k];
          combined.push(namespace);
          breadcrumbs.push({
            href: this.referencer.getLink(combined.join('.'), assetPath),
            name: namespace
          });
        }
        breadcrumbs.push({
          name: mixin.getName()
        });
        isIncludedIn = function(clazz) {
          var _ref1;

          if (clazz != null ? (_ref1 = clazz.doc) != null ? _ref1.includeMixins : void 0 : void 0) {
            return _.contains(clazz.doc.includeMixins, mixin.getMixinName());
          }
        };
        isExtendedFrom = function(clazz) {
          var _ref1;

          if (clazz != null ? (_ref1 = clazz.doc) != null ? _ref1.extendMixins : void 0 : void 0) {
            return _.contains(clazz.doc.extendMixins, mixin.getMixinName());
          }
        };
        _results.push(this.templater.render('mixin', {
          path: assetPath,
          mixinData: mixin.toJSON(),
          includedIn: _.map(_.filter(this.parser.classes, isIncludedIn), function(c) {
            return c.getClassName();
          }),
          extendedIn: _.map(_.filter(this.parser.classes, isExtendedFrom), function(c) {
            return c.getClassName();
          }),
          methods: _.map(mixin.getMethods(), function(m) {
            return m.toJSON();
          }),
          constants: _.map(_.filter(mixin.getVariables(), function(variable) {
            return variable.isConstant();
          }), function(m) {
            return m.toJSON();
          }),
          breadcrumbs: breadcrumbs
        }, "mixins/" + (mixin.getFullName().replace(/\./g, '/')) + ".html"));
      }
      return _results;
    };

    Generator.prototype.generateFiles = function() {
      var assetPath, breadcrumbs, combined, file, p, segment, _i, _j, _k, _len, _len1, _len2, _ref, _results,
        _this = this;

      _ref = this.parser.files;
      _results = [];
      for (_i = 0, _len = _ref.length; _i < _len; _i++) {
        file = _ref[_i];
        p = _.compact(file.getPath().split('/'));
        assetPath = '../';
        for (_j = 0, _len1 = p.length; _j < _len1; _j++) {
          segment = p[_j];
          assetPath += '../';
        }
        breadcrumbs = [
          {
            href: "" + assetPath + "class_index.html",
            name: 'Index'
          }
        ];
        if (this.options.readme) {
          breadcrumbs.unshift({
            href: "" + assetPath + this.options.readme + ".html",
            name: this.options.name
          });
        }
        if (this.options.homepage) {
          breadcrumbs.unshift(this.options.homepage);
        }
        combined = [];
        for (_k = 0, _len2 = p.length; _k < _len2; _k++) {
          segment = p[_k];
          combined.push(segment);
          breadcrumbs.push({
            href: this.referencer.getLink(combined.join('.'), assetPath),
            name: segment
          });
        }
        breadcrumbs.push({
          name: file.getFileName()
        });
        _results.push(this.templater.render('file', {
          path: assetPath,
          filename: file.getFileName(),
          filepath: file.getPath(),
          classname: file.getClassName(),
          methods: _.map(file.getMethods(), function(m) {
            return _this.referencer.resolveDoc(m.toJSON(), file, assetPath);
          }),
          constants: _.map(_.filter(file.getVariables(), function(variable) {
            return variable.isConstant();
          }), function(m) {
            return _this.referencer.resolveDoc(m.toJSON(), file, assetPath);
          }),
          breadcrumbs: breadcrumbs
        }, "files/" + (file.getFullName()) + ".html"));
      }
      return _results;
    };

    Generator.prototype.generateExtras = function() {
      var assetPath, breadcrumbs, content, error, extra, filename, numSlashes, slash, _i, _j, _len, _ref, _results;

      _ref = _.union([this.options.readme], this.options.extras);
      _results = [];
      for (_i = 0, _len = _ref.length; _i < _len; _i++) {
        extra = _ref[_i];
        try {
          if ((fs.existsSync || path.existsSync)(extra)) {
            content = fs.readFileSync(extra, 'utf-8');
            if (/\.(markdown|md)$/.test(extra)) {
              content = Markdown.convert(content);
            }
            numSlashes = extra.split('/').length - 1;
            assetPath = '';
            for (slash = _j = 0; 0 <= numSlashes ? _j < numSlashes : _j > numSlashes; slash = 0 <= numSlashes ? ++_j : --_j) {
              assetPath += '../';
            }
            filename = "" + extra + ".html";
            breadcrumbs = [
              {
                href: "" + assetPath + "class_index.html",
                name: 'Index'
              }, {
                href: "File: " + filename,
                name: extra
              }
            ];
            if (this.options.readme) {
              breadcrumbs.unshift({
                href: "" + assetPath + this.options.readme + ".html",
                name: this.options.name
              });
            }
            if (this.options.homepage) {
              breadcrumbs.unshift(this.options.homepage);
            }
            _results.push(this.templater.render('extra', {
              path: assetPath,
              filename: extra,
              content: content,
              breadcrumbs: breadcrumbs
            }, filename));
          } else {
            _results.push(void 0);
          }
        } catch (_error) {
          error = _error;
          _results.push(console.log("[ERROR] Cannot generate extra file " + extra + ": " + error));
        }
      }
      return _results;
    };

    Generator.prototype.generateClassMixinFileExtraIndex = function() {
      var char, classes, code, files, mixins, sortedClasses, sortedFiles, x, _i, _j, _k, _l, _len, _len1, _len2;

      sortedClasses = {};
      sortedFiles = {};
      for (code = _i = 97; _i <= 122; code = ++_i) {
        char = String.fromCharCode(code);
        classes = _.filter(this.parser.classes, function(clazz) {
          return clazz.getName().toLowerCase()[0] === char;
        });
        mixins = _.filter(this.parser.mixins, function(mixin) {
          return mixin.getName().toLowerCase()[0] === char;
        });
        files = _.filter(this.parser.files, function(file) {
          return file.getFileName().toLowerCase()[0] === char;
        });
        if (classes.length + mixins.length > 0) {
          sortedClasses[char] = [];
          if (!_.isEmpty(classes)) {
            for (_j = 0, _len = classes.length; _j < _len; _j++) {
              x = classes[_j];
              sortedClasses[char].push(x);
            }
          }
          if (!_.isEmpty(mixins)) {
            for (_k = 0, _len1 = mixins.length; _k < _len1; _k++) {
              x = mixins[_k];
              sortedClasses[char].push(x);
            }
          }
        }
        if (files.length > 0) {
          sortedFiles[char] = [];
          for (_l = 0, _len2 = files.length; _l < _len2; _l++) {
            x = files[_l];
            sortedFiles[char].push(x);
          }
        }
      }
      return this.templater.render('index', {
        path: '',
        classes: sortedClasses,
        files: sortedFiles,
        extras: _.union([this.options.readme], this.options.extras.sort()),
        breadcrumbs: []
      }, 'class_index.html');
    };

    Generator.prototype.generateClassAndMixinLists = function() {
      var classes, clazz, mixin, mixins, traverse, _i, _j, _len, _len1, _ref, _ref1;

      classes = [];
      mixins = [];
      traverse = function(entity, children, section) {
        var child, namespace, namespaces;

        if (entity.getNamespace()) {
          namespaces = entity.getNamespace().split('.');
          while (namespace = namespaces.shift()) {
            child = _.find(children, function(c) {
              return c.name === namespace;
            });
            if (!child) {
              child = {
                name: namespace
              };
              children.push(child);
            }
            child.children || (child.children = []);
            children = child.children;
          }
        }
        return children.push({
          name: entity.getName(),
          href: "" + section + "/" + (entity.getFullName().replace(/\./g, '/')) + ".html",
          parent: typeof entity.getParentClassName === "function" ? entity.getParentClassName() : void 0,
          namespace: entity.getNamespace()
        });
      };
      _ref = this.parser.classes;
      for (_i = 0, _len = _ref.length; _i < _len; _i++) {
        clazz = _ref[_i];
        traverse(clazz, classes, 'classes');
      }
      _ref1 = this.parser.mixins;
      for (_j = 0, _len1 = _ref1.length; _j < _len1; _j++) {
        mixin = _ref1[_j];
        traverse(mixin, mixins, 'mixins');
      }
      this.templater.render('class_list', {
        path: '',
        classes: classes
      }, 'class_list.html');
      return this.templater.render('mixin_list', {
        path: '',
        mixins: mixins
      }, 'mixin_list.html');
    };

    Generator.prototype.generateMethodList = function() {
      var methods, nonconstructors;

      nonconstructors = _.filter(this.parser.getAllMethods(), function(m) {
        return m.getName() !== 'constructor';
      });
      methods = _.map(nonconstructors, function(method) {
        var href, _ref;

        href = (function() {
          switch (method.entity.constructor.name) {
            case 'Class':
              return "classes/" + (method.entity.getFullName().replace(/\./g, '/')) + ".html#" + (method.getName()) + "-" + (method.getType());
            case 'Mixin':
              return "mixins/" + (method.entity.getFullName().replace(/\./g, '/')) + ".html#" + (method.getName()) + "-" + (method.getType());
            case 'File':
              return "files/" + (method.entity.getFullName()) + ".html#" + (method.getName()) + "-" + (method.getType());
          }
        })();
        return {
          path: '',
          name: method.getName(),
          href: href,
          classname: method.entity.getFullName(),
          deprecated: (_ref = method.doc) != null ? _ref.deprecated : void 0,
          type: method.type
        };
      });
      return this.templater.render('method_list', {
        methods: _.sortBy(methods, function(method) {
          return method.name;
        })
      }, 'method_list.html');
    };

    Generator.prototype.generateFileList = function() {
      var file, files, traverse, _i, _len, _ref;

      files = [];
      traverse = function(entity, children) {
        var child, segment, segments;

        if (entity.getPath()) {
          segments = entity.getPath().split('/');
          while (segment = segments.shift()) {
            child = _.find(children, function(c) {
              return c.name === segment;
            });
            if (!child) {
              child = {
                name: segment
              };
              children.push(child);
            }
            child.children || (child.children = []);
            children = child.children;
          }
        }
        return children.push({
          name: entity.getFileName(),
          href: "files/" + (entity.getFullName()) + ".html",
          path: entity.getPath()
        });
      };
      _ref = this.parser.files;
      for (_i = 0, _len = _ref.length; _i < _len; _i++) {
        file = _ref[_i];
        traverse(file, files);
      }
      return this.templater.render('file_list', {
        path: '',
        files: files
      }, 'file_list.html');
    };

    Generator.prototype.generateExtraList = function() {
      return this.templater.render('extra_list', {
        path: '',
        extras: _.union([this.options.readme], this.options.extras.sort())
      }, 'extra_list.html');
    };

    Generator.prototype.copyAssets = function() {
      this.copy(path.join(__dirname, '..', 'theme', 'default', 'assets', 'biscotto.css'), path.join(this.options.output, 'assets', 'biscotto.css'));
      return this.copy(path.join(__dirname, '..', 'theme', 'default', 'assets', 'biscotto.js'), path.join(this.options.output, 'assets', 'biscotto.js'));
    };

    Generator.prototype.copy = function(from, to) {
      var dir;

      dir = path.dirname(to);
      return mkdirp(dir, function(err) {
        if (err) {
          return console.error("[ERROR] Cannot create directory " + dir + ": " + err);
        } else {
          from = fs.createReadStream(from);
          to = fs.createWriteStream(to);
          return from.pipe(to);
        }
      });
    };

    Generator.prototype.generateSearchData = function(file) {
      var clazz, destinationFolder, f, method, mixin, search, _i, _j, _k, _l, _len, _len1, _len2, _len3, _len4, _len5, _len6, _m, _n, _o, _ref, _ref1, _ref2, _ref3, _ref4, _ref5, _ref6;

      search = [];
      _ref = this.parser.classes;
      for (_i = 0, _len = _ref.length; _i < _len; _i++) {
        clazz = _ref[_i];
        search.push({
          t: clazz.getClassName(),
          p: "classes/" + (clazz.getClassName().replace(/\./g, '/')) + ".html"
        });
        _ref1 = clazz.getMethods();
        for (_j = 0, _len1 = _ref1.length; _j < _len1; _j++) {
          method = _ref1[_j];
          search.push({
            t: method.getShortSignature(),
            h: clazz.getClassName(),
            p: "classes/" + (clazz.getClassName().replace(/\./g, '/')) + ".html#" + method.name + "-" + method.type
          });
        }
      }
      _ref2 = this.parser.mixins;
      for (_k = 0, _len2 = _ref2.length; _k < _len2; _k++) {
        mixin = _ref2[_k];
        search.push({
          t: mixin.getMixinName(),
          p: "mixins/" + (mixin.getFullName().replace(/\./g, '/')) + ".html"
        });
        _ref3 = mixin.getMethods();
        for (_l = 0, _len3 = _ref3.length; _l < _len3; _l++) {
          method = _ref3[_l];
          search.push({
            t: method.getShortSignature(),
            p: "mixins/" + (mixin.getFullName().replace(/\./g, '/')) + ".html#" + method.name + "-" + method.type,
            h: mixin.getMixinName()
          });
        }
      }
      _ref4 = this.parser.files;
      for (_m = 0, _len4 = _ref4.length; _m < _len4; _m++) {
        f = _ref4[_m];
        search.push({
          t: f.getFileName(),
          p: "files/" + (f.getFullName()) + ".html"
        });
        _ref5 = f.getMethods();
        for (_n = 0, _len5 = _ref5.length; _n < _len5; _n++) {
          method = _ref5[_n];
          search.push({
            t: method.getShortSignature(),
            p: "files/" + (f.getFullName()) + ".html#" + method.name + "-" + method.type,
            h: f.getFileName()
          });
        }
      }
      _ref6 = _.union([this.options.readme], this.options.extras.sort());
      for (_o = 0, _len6 = _ref6.length; _o < _len6; _o++) {
        f = _ref6[_o];
        search.push({
          t: f,
          p: "" + f + ".html"
        });
      }
      if (file) {
        return file('assets/search_data.js', 'window.searchData = ' + JSON.stringify(search));
      } else {
        if (!this.options.noOutput) {
          destinationFolder = path.join(this.options.output, 'assets');
          return mkdirp(destinationFolder, function(err) {
            var destinationFile;

            if (err) {
              return console.error("[ERROR] Cannot create directory " + dir + ": " + err);
            } else {
              destinationFile = path.join(destinationFolder, 'search_data.js');
              return fs.writeFile(destinationFile, 'window.searchData = ' + JSON.stringify(search), function(err) {
                if (err) {
                  return console.error("[ERROR] Cannot write search data: ", err);
                }
              });
            }
          });
        }
      }
    };

    return Generator;

  })();

}).call(this);
