// Generated by CoffeeScript 1.6.2
(function() {
  var Doc, Markdown, Node, Referencer, marked, _,
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  Node = require('./node');

  Markdown = require('../util/markdown');

  Referencer = require('../util/referencer');

  marked = require('marked');

  _ = require('underscore');

  _.str = require('underscore.string');

  module.exports = Doc = (function(_super) {
    __extends(Doc, _super);

    function Doc(node, options) {
      var error;

      this.node = node;
      this.options = options;
      try {
        if (this.node) {
          this.parseBlock(this.leftTrimBlock(this.node.comment.replace(/\u0091/gm, '').split('\n')));
        }
      } catch (_error) {
        error = _error;
        if (this.options.verbose) {
          console.warn('Create doc error:', this.node, error);
        }
      }
    }

    Doc.prototype.hasComment = function() {
      return !_.str.isBlank(this.comment);
    };

    Doc.prototype.leftTrimBlock = function(lines) {
      var line, minimalTrim, trimMap;

      trimMap = _.map(lines, function(line) {
        if (line.length === 0) {
          return void 0;
        } else {
          return line.length - _.str.ltrim(line).length;
        }
      });
      minimalTrim = _.min(_.without(trimMap, void 0));
      if (minimalTrim > 0 && minimalTrim < Infinity) {
        lines = (function() {
          var _i, _len, _results;

          _results = [];
          for (_i = 0, _len = lines.length; _i < _len; _i++) {
            line = lines[_i];
            line = line.substring(minimalTrim, line.length);
            _results.push(line);
          }
          return _results;
        })();
      }
      return lines;
    };

    Doc.prototype.parseBlock = function(lines) {
      var comment, current, delegationMatch, info_block, sections, sentence, text;

      comment = [];
      if (lines === void 0) {
        return;
      }
      sections = lines.join("\n").split("\n\n");
      info_block = this.parse_description(sections.shift());
      text = info_block.description;
      this.status = info_block.status;
      delegationMatch = text.match(/\{Delegates to: (.+?)\}/);
      if (delegationMatch && (this.delegation = delegationMatch[1])) {
        return;
      }
      current = sections.shift();
      while (current) {
        if (/^\w+\s+\-/.test(current)) {
          this.params || (this.params = []);
          this.params = this.parse_arguments(current);
        } else if (/^\s*Examples/.test(current)) {
          this.examples || (this.examples = []);
          this.examples = this.parse_examples(current, sections);
        } else if (/^\s*Returns/.test(current)) {
          this.returnValue || (this.returnValue = []);
          this.returnValue = this.parse_returns(current);
        } else {
          text = text.concat("\n\n" + current);
        }
        current = sections.shift();
      }
      this.comment = Markdown.convert(text);
      sentence = /((?:.|\n)*?[.#][\s$])/.exec(text);
      if (sentence) {
        sentence = sentence[1].replace(/\s*#\s*$/, '');
      }
      return this.summary = Markdown.convert(_.str.clean(sentence || text), true);
    };

    Doc.prototype.parse_description = function(section) {
      var md;

      if (md = /([A-Z]\w+)\:(.*)/.exec(section)) {
        return {
          status: md[1],
          description: _.str.strip(md[2])
        };
      } else {
        return {
          description: _.str.strip(section)
        };
      }
    };

    Doc.prototype.parse_examples = function(section, sections) {
      var examples, lines;

      examples = [];
      section = _.str.strip(section.replace(/Examples/, ''));
      if (!_.isEmpty(section)) {
        examples.push(section);
      }
      while (_.first(sections)) {
        lines = sections.shift().split("\n");
        examples.push(this.deindent(lines).join("\n"));
      }
      return examples;
    };

    Doc.prototype.parse_returns = function(section) {
      var current, lines, returns;

      returns = [];
      current = [];
      lines = section.split("\n");
      _.each(lines, function(line) {
        line = _.str.trim(line);
        if (/^Returns/.test(line)) {
          returns.push({
            type: Referencer.getLinkMatch(line),
            desc: Markdown.convert(line).replace(/<\/?p>/g, "")
          });
          return current = returns;
        } else if (/^\S+/.test(line)) {
          return _.last(returns).desc = _.last(returns).desc.concat("\n" + _.str.strip(line));
        }
      });
      return returns;
    };

    Doc.prototype.parse_arguments = function(section) {
      var args, last_indent;

      args = [];
      last_indent = null;
      _.each(section.split("\n"), function(line) {
        var arg, desc, indent, key, keyDesc, param, param_match, stripped_line, _base;

        if (!_.isEmpty(line)) {
          indent = line.match(/^(\s*)/)[0].length;
          stripped_line = _.str.strip(line);
          if (last_indent !== null && indent >= last_indent && (indent !== 0) && stripped_line.match(/^\w+:/) === null) {
            _.last(args).desc += " " + Markdown.convert(stripped_line).replace(/<\/?p>/g, "");
          } else {
            arg = line.split(" - ");
            param = _.str.strip(arg[0]);
            desc = Markdown.convert(_.str.strip(arg[1])).replace(/<\/?p>/g, "");
            param_match = param.match(/^\w+:/);
            if (param_match && _.str.endsWith(param_match[0], ":")) {
              (_base = _.last(args)).options || (_base.options = []);
              key = param.split(":");
              keyDesc = _.str.strip(key[1]);
              _.last(args).options.push({
                name: key[0],
                desc: Markdown.convert(keyDesc).replace(/<\/?p>/g, ""),
                type: Referencer.getLinkMatch(line)
              });
            } else {
              args.push({
                name: param,
                desc: desc,
                type: Referencer.getLinkMatch(line)
              });
            }
          }
          return last_indent = indent;
        }
      });
      return args;
    };

    Doc.prototype.deindent = function(lines) {
      var space, spaces;

      spaces = _.map(lines, function(line) {
        var md;

        if (_.isEmpty(_.str.strip(line))) {
          return line;
        }
        md = line.match(/^(\s*)/);
        if (md) {
          return md[1].length;
        } else {
          return null;
        }
      });
      spaces = _.compact(spaces);
      space = _.min(spaces) || 0;
      return _.map(lines, function(line) {
        if (_.isEmpty(line)) {
          return _.str.strip(line);
        } else {
          return line.slice(space);
        }
      });
    };

    Doc.prototype.toJSON = function() {
      var json;

      if (this.node) {
        json = {
          includes: this.includeMixins,
          "extends": this.extendMixins,
          concerns: this.concerns,
          abstract: this.abstract,
          "private": this["private"],
          deprecated: this.deprecated,
          version: this.version,
          since: this.since,
          examples: this.examples,
          todos: this.todos,
          notes: this.notes,
          authors: this.authors,
          copyright: this.copyright,
          comment: this.comment,
          summary: this.summary,
          status: this.status,
          params: this.params,
          options: this.paramsOptions,
          delegation: this.delegation,
          see: this.see,
          returnValue: this.returnValue,
          throwValue: this.throwValue,
          overloads: this.overloads,
          methods: this.methods,
          property: this.property
        };
        return json;
      }
    };

    return Doc;

  })(Node);

}).call(this);
