// Generated by CoffeeScript 1.6.2
(function() {
  var Class, CoffeeScript, File, Mixin, Parser, VirtualMethod, fs, whitespace, _;

  fs = require('fs');

  _ = require('underscore');

  _.str = require('underscore.string');

  CoffeeScript = require('coffee-script');

  File = require('./nodes/file');

  Class = require('./nodes/class');

  Mixin = require('./nodes/mixin');

  VirtualMethod = require('./nodes/virtual_method');

  whitespace = require('./util/text').whitespace;

  module.exports = Parser = (function() {
    function Parser(options) {
      this.options = options;
      this.files = [];
      this.classes = [];
      this.mixins = [];
      this.fileCount = 0;
      this.globalStatus = "Public";
      this.classMemberRegex = "";
    }

    Parser.prototype.parseFile = function(file) {
      this.parseContent(fs.readFileSync(file, 'utf8'), file);
      return this.fileCount += 1;
    };

    Parser.prototype.parseContent = function(content, file) {
      var entities, error, fileClass, root,
        _this = this;

      if (file == null) {
        file = '';
      }
      this.previousNodes = [];
      this.globalStatus = "Public";
      entities = {
        clazz: function(node) {
          var _ref, _ref1;

          return node.constructor.name === 'Class' && (((_ref = node.variable) != null ? (_ref1 = _ref.base) != null ? _ref1.value : void 0 : void 0) != null);
        },
        mixin: function(node) {
          var _ref, _ref1;

          return node.constructor.name === 'Assign' && (((_ref = node.value) != null ? (_ref1 = _ref.base) != null ? _ref1.properties : void 0 : void 0) != null);
        }
      };
      if (!this.options.cautious) {
        content = this.convertComments(content);
      }
      try {
        root = CoffeeScript.nodes(content);
      } catch (_error) {
        error = _error;
        if (this.options.debug) {
          console.log('Parsed CoffeeScript source:\n%s', content);
        }
        throw error;
      }
      fileClass = new File(root, file, this.options);
      if (!fileClass.isEmpty()) {
        this.files.push(fileClass);
      }
      this.linkAncestors(root);
      root.traverseChildren(true, function(child) {
        var clazz, condition, doc, entity, mixin, name, node, p, previous, type, _i, _len, _ref, _ref1;

        entity = false;
        for (type in entities) {
          condition = entities[type];
          if (entities.hasOwnProperty(type)) {
            if (condition(child)) {
              entity = type;
            }
          }
        }
        if (entity) {
          previous = _this.previousNodes[_this.previousNodes.length - 1];
          switch (previous != null ? previous.constructor.name : void 0) {
            case 'Comment':
              doc = previous;
              break;
            case 'Literal':
              if (previous.value === 'exports') {
                node = _this.previousNodes[_this.previousNodes.length - 6];
                if ((node != null ? node.constructor.name : void 0) === 'Comment') {
                  doc = node;
                }
              }
          }
          if (entity === 'mixin') {
            name = [child.variable.base.value];
            _ref = child.variable.properties;
            for (_i = 0, _len = _ref.length; _i < _len; _i++) {
              p = _ref[_i];
              name.push((_ref1 = p.name) != null ? _ref1.value : void 0);
            }
            if (name.indexOf(void 0) === -1) {
              mixin = new Mixin(child, file, _this.options, doc);
              if ((mixin.doc.mixin != null) && (_this.options["private"] || !mixin.doc["private"])) {
                _this.mixins.push(mixin);
              }
            }
          }
          if (entity === 'clazz') {
            clazz = new Class(child, file, _this.options, doc);
            if (!_this.options["private"] && clazz.doc.status === "Private") {

            } else if (!_this.options.internal && clazz.doc.status === "Internal") {

            } else {
              _this.classes.push(clazz);
            }
          }
        }
        _this.previousNodes.push(child);
        return true;
      });
      return root;
    };

    Parser.prototype.convertComments = function(content) {
      var blockComment, c, comment, commentLine, globalStatusBlock, inBlockComment, inComment, indentComment, line, member, result, _i, _j, _len, _len1, _ref, _ref1, _ref2;

      result = [];
      comment = [];
      inComment = false;
      inBlockComment = false;
      indentComment = 0;
      _ref = content.split('\n');
      for (_i = 0, _len = _ref.length; _i < _len; _i++) {
        line = _ref[_i];
        globalStatusBlock = false;
        if (globalStatusBlock = /^\s*#{3} (\w+).+?#{3}/.exec(line)) {
          this.globalStatus = globalStatusBlock[1];
        }
        blockComment = /^\s*#{3,}/.exec(line) && !/^\s*#{3,}.+#{3,}/.exec(line);
        if (blockComment || inBlockComment) {
          if (blockComment) {
            inBlockComment = !inBlockComment;
          }
          result.push(line);
        } else {
          commentLine = /^(\s*#)\s?(\s*.*)/.exec(line);
          if (commentLine) {
            if (inComment) {
              comment.push((_ref1 = commentLine[2]) != null ? _ref1.replace(/#/g, "\u0091#") : void 0);
            } else {
              if (!/^\s*\w+:/.test(commentLine[2])) {
                commentLine[2] = this.globalStatus + ": " + commentLine[2];
              }
              inComment = true;
              indentComment = commentLine[1].length - 1;
              comment.push(whitespace(indentComment) + '###');
              comment.push((_ref2 = commentLine[2]) != null ? _ref2.replace(/#/g, "\u0091#") : void 0);
            }
          } else {
            if (inComment) {
              inComment = false;
              comment.push(whitespace(indentComment) + '###');
              if (/(class\s*[$A-Za-z_\x7f-\uffff][$\w\x7f-\uffff]*|^\s*[$A-Za-z_\x7f-\uffff][$\w\x7f-\uffff.]*\s+\=|[$A-Za-z_\x7f-\uffff][$\w\x7f-\uffff]*\s*:\s*(\(.*\)\s*)?[-=]>|@[A-Za-z_\x7f-\uffff][$\w\x7f-\uffff]*\s*=\s*(\(.*\)\s*)?[-=]>|[$A-Za-z_\x7f-\uffff][\.$\w\x7f-\uffff]*\s*=\s*(\(.*\)\s*)?[-=]>|^\s*@[$A-Z_][A-Z_]*)|^\s*[$A-Za-z_\x7f-\uffff][$\w\x7f-\uffff]*:\s*\S+/.exec(line)) {
                for (_j = 0, _len1 = comment.length; _j < _len1; _j++) {
                  c = comment[_j];
                  result.push(c);
                }
              }
              comment = [];
            }
            member = /(class\s*[$A-Za-z_\x7f-\uffff][$\w\x7f-\uffff]*|^\s*[$A-Za-z_\x7f-\uffff][$\w\x7f-\uffff.]*\s+\=|[$A-Za-z_\x7f-\uffff][$\w\x7f-\uffff]*\s*:\s*(\(.*\)\s*)?[-=]>|@[A-Za-z_\x7f-\uffff][$\w\x7f-\uffff]*\s*=\s*(\(.*\)\s*)?[-=]>|[$A-Za-z_\x7f-\uffff][\.$\w\x7f-\uffff]*\s*=\s*(\(.*\)\s*)?[-=]>|^\s*@[$A-Z_][A-Z_]*)|^\s*[$A-Za-z_\x7f-\uffff][$\w\x7f-\uffff]*:\s*\S+/.exec(line);
            if (member && _.str.isBlank(_.last(result))) {
              indentComment = /^(\s*)/.exec(line);
              if (indentComment) {
                indentComment = indentComment[1];
              } else {
                indentComment = "";
              }
              result.push("" + indentComment + "###");
              result.push("" + this.globalStatus + ":");
              result.push("" + indentComment + "###");
            }
            result.push(line);
          }
        }
      }
      return result.join('\n');
    };

    Parser.prototype.linkAncestors = function(node) {
      var _this = this;

      return node.eachChild(function(child) {
        child.ancestor = node;
        return _this.linkAncestors(child);
      });
    };

    Parser.prototype.getAllMethods = function() {
      var clazz, file, mixin, _i, _j, _k, _len, _len1, _len2, _ref, _ref1, _ref2;

      if (!this.methods) {
        this.methods = [];
        _ref = this.files;
        for (_i = 0, _len = _ref.length; _i < _len; _i++) {
          file = _ref[_i];
          this.methods = _.union(this.methods, file.getMethods());
        }
        _ref1 = this.classes;
        for (_j = 0, _len1 = _ref1.length; _j < _len1; _j++) {
          clazz = _ref1[_j];
          this.methods = _.union(this.methods, clazz.getMethods());
        }
        _ref2 = this.mixins;
        for (_k = 0, _len2 = _ref2.length; _k < _len2; _k++) {
          mixin = _ref2[_k];
          this.methods = _.union(this.methods, mixin.getMethods());
        }
      }
      return this.methods;
    };

    Parser.prototype.getAllVariables = function() {
      var clazz, file, mixin, _i, _j, _k, _len, _len1, _len2, _ref, _ref1, _ref2;

      if (!this.variables) {
        this.variables = [];
      }
      _ref = this.files;
      for (_i = 0, _len = _ref.length; _i < _len; _i++) {
        file = _ref[_i];
        this.variables = _.union(this.variables, file.getVariables());
      }
      _ref1 = this.classes;
      for (_j = 0, _len1 = _ref1.length; _j < _len1; _j++) {
        clazz = _ref1[_j];
        this.variables = _.union(this.variables, clazz.getVariables());
      }
      _ref2 = this.mixins;
      for (_k = 0, _len2 = _ref2.length; _k < _len2; _k++) {
        mixin = _ref2[_k];
        this.methods = _.union(this.methods, mixin.getMethods());
      }
      return this.variables;
    };

    Parser.prototype.showResult = function(generator) {
      var classCount, constantCount, constants, documented, fileCount, maxCountLength, maxNoDocLength, methodCount, methodsToCount, mixinCount, noDocClass, noDocClassNames, noDocClasses, noDocClassesLength, noDocConstants, noDocMethod, noDocMethodNames, noDocMethods, noDocMethodsLength, prevFileName, stats, totalFound, totalNoDoc, _i, _j, _len, _len1;

      fileCount = this.files.length;
      classCount = this.classes.length;
      noDocClasses = _.filter(this.classes, function(clazz) {
        return !clazz.getDoc().hasComment();
      });
      noDocClassesLength = noDocClasses.length;
      mixinCount = this.mixins.length;
      methodsToCount = _.filter(this.getAllMethods(), function(method) {
        return !(method instanceof VirtualMethod);
      });
      methodCount = methodsToCount.length;
      noDocMethods = _.filter(methodsToCount, function(method) {
        return !method.getDoc().hasComment();
      });
      noDocMethodsLength = noDocMethods.length;
      constants = _.filter(this.getAllVariables(), function(variable) {
        return variable.isConstant();
      });
      constantCount = constants.length;
      noDocConstants = _.filter(constants, function(constant) {
        return !constant.getDoc().hasComment();
      }).length;
      totalFound = classCount + methodCount + constantCount;
      totalNoDoc = noDocClassesLength + noDocMethodsLength + noDocConstants;
      documented = 100 - 100 / (classCount + methodCount + constantCount) * (noDocClassesLength + noDocMethodsLength + noDocConstants);
      maxCountLength = String(_.max([fileCount, mixinCount, classCount, methodCount, constantCount], function(count) {
        return String(count).length;
      })).length + 6;
      maxNoDocLength = String(_.max([noDocClassesLength, noDocMethodsLength, noDocConstants], function(count) {
        return String(count).length;
      })).length;
      stats = "Parsed files:    " + (_.str.pad(this.fileCount, maxCountLength)) + "\nClasses:         " + (_.str.pad(classCount, maxCountLength)) + " (" + (_.str.pad(noDocClassesLength, maxNoDocLength)) + " undocumented)\nMixins:          " + (_.str.pad(mixinCount, maxCountLength)) + "\nNon-Class files: " + (_.str.pad(fileCount, maxCountLength)) + "\nMethods:         " + (_.str.pad(methodCount, maxCountLength)) + " (" + (_.str.pad(noDocMethodsLength, maxNoDocLength)) + " undocumented)\nConstants:       " + (_.str.pad(constantCount, maxCountLength)) + " (" + (_.str.pad(noDocConstants, maxNoDocLength)) + " undocumented)\n " + (_.str.sprintf('%.2f', documented)) + "% documented (" + totalFound + " total, " + totalNoDoc + " with no doc)\n " + generator.referencer.errors + " errors";
      if (this.options.missing) {
        noDocClassNames = [];
        for (_i = 0, _len = noDocClasses.length; _i < _len; _i++) {
          noDocClass = noDocClasses[_i];
          noDocClassNames.push(noDocClass.className);
        }
        noDocMethodNames = [];
        prevFileName = "";
        for (_j = 0, _len1 = noDocMethods.length; _j < _len1; _j++) {
          noDocMethod = noDocMethods[_j];
          if (prevFileName !== noDocMethod.entity.fileName) {
            prevFileName = noDocMethod.entity.fileName;
            noDocMethodNames.push("\nIn " + prevFileName + ":");
          }
          noDocMethodNames.push(noDocMethod.shortSignature);
        }
        stats += "\n";
        if (noDocClassNames.length > 0) {
          stats += "\nClasses missing docs:\n\n" + (noDocClassNames.join('\n'));
        }
        if (noDocMethodNames.length > 0) {
          stats += "\n\nMethods missing docs:\n" + (noDocMethodNames.join('\n'));
        }
      }
      console.log(stats);
      if (this.options.json && this.options.json.length) {
        return fs.writeFileSync(this.options.json, JSON.stringify(this.toJSON(), null, "    "));
      }
    };

    Parser.prototype.toJSON = function() {
      var clazz, file, json, mixin, _i, _j, _k, _len, _len1, _len2, _ref, _ref1, _ref2;

      json = [];
      _ref = this.files;
      for (_i = 0, _len = _ref.length; _i < _len; _i++) {
        file = _ref[_i];
        json.push(file.toJSON());
      }
      _ref1 = this.classes;
      for (_j = 0, _len1 = _ref1.length; _j < _len1; _j++) {
        clazz = _ref1[_j];
        json.push(clazz.toJSON());
      }
      _ref2 = this.mixins;
      for (_k = 0, _len2 = _ref2.length; _k < _len2; _k++) {
        mixin = _ref2[_k];
        json.push(mixin.toJSON());
      }
      return json;
    };

    return Parser;

  })();

}).call(this);
